<?php
/**
 * Vine MA Plugin
 * @copyright   Copyright (C) 2020 Vine Ltd. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
*/

defined('_JEXEC') or die;

define("VINEHOST", "https://vine.eu");

class plgSystemVineMA extends JPlugin
{
    /**
    * Load the language file on instantiation.
    *
    * @var    boolean
    * @since  3.1
    */
    protected $autoloadLanguage = true;

    public function onAfterRender()
    {
        $app = JFactory::getApplication();
        if ($app->isClient('site')) {
            $body = $app->getBody();
            $orgId = $this->params->get('organization_id', 1);
            $src = VINEHOST."/track/t.js?sg={$orgId}";
            $srcac = VINEHOST."/track/allowCookies.js";
            $scripts = '<head><script type="text/javascript" src="'.$src.'" data-cookieconsent="ignore"></script><script type="text/plain" src="'.$srcac.'" data-cookieconsent="marketing" async></script>';
            $body = str_replace('<head>', $scripts , $body);
            $app->setBody($body);
        }
    }

    public function onContentPrepare($context, &$article, &$params, $page = 0)
    {
        $webforms = $this->params->get('web_forms');
        $webFormsRefreshed = false;
        if($webforms == null) {
            $webforms = $this->vine_ma_get_web_forms();
            $webFormsRefreshed = true;
        }
        $that = $this;
        $pattern = '/{VineForm(\s|&nbsp;)([0-9]+?)}/';
        $content = "";
        $content .= (string)$article->text;
        $newtext = preg_replace_callback($pattern, function($matches) use(&$webforms, &$webFormsRefreshed, &$that) {
                $targetkey = array_search($matches[2], array_column($webforms, 'id'));
                if($targetkey === false && $webFormsRefreshed === false) {
                      $webforms = $that->vine_ma_get_web_forms();
                      $webFormsRefreshed = true;
                      $targetkey = array_search($matches[2], array_column($webforms, 'id'));
                }
                if($targetkey === false)
                    return 'Vine Form (' . $matches[2] . ') is not found!';
                else {
                    $form = $webforms[$targetkey];
                    $formid = is_array($form) ? $form['id'] : $form->id;
                    $formtype = is_array($form) ? $form['type'] : $form->type;

                    if($formtype == 0)
                        return "<div class=\"VineForm\" data-form-id=\"{$formid}\"></div>";
                    else
                        return "<div class=\"VineForm B{$formid}\" data-bot-id=\"{$formid}\"></div>";
                }
            }, $content);
        $article->text = $newtext;
        return true;
    }

    private function vine_ma_get_web_forms() {
        $apikey = $this->params->get('api_key');
        $token = null;
        if( $apikey != null) {
            $token = $this->vine_ma_get_authtoken($apikey);
            if( $token == null ) 
                return array();
        }
        $url=VINEHOST."/api/rest/2.0/VS_TRACK_FORM([FORMTYPE]<>1%20and%20[FORMTYPE]<>2%20and%20[FORMTYPE]<>3%20and%20[FORMTYPE]<>4)?order=name&\$authtoken={$token}";
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, 120);
        $html = curl_exec($ch);
        $webforms = array();
        if(!curl_errno($ch) && $html != 'Unauthorized') {
            libxml_use_internal_errors(true);
            $dom = new DOMDocument("1.0", "UTF-8");
            $dom->strictErrorChecking = false;
            $dom->validateOnParse = false;
            $dom->recover = true;
            $dom->loadXML($html);
            $xml = simplexml_import_dom($dom);
            libxml_clear_errors();
            libxml_use_internal_errors(false);
            
            foreach($xml->xpath('//m:properties') as $event) {
                    array_push($webforms, array(
                    'id' => (string)$event->xpath('d:ID')[0],
                    'name' => (string)$event->xpath('d:NAME')[0],
                    'type' => (string)$event->xpath('d:FORMTYPE')[0],
                ));
            }
        } 
        curl_close($ch);
        //store new webforms to DB
        $table = new JTableExtension(JFactory::getDbo());
        $table->load(array('element' => 'vinema'));
        $this->params->set('web_forms', $webforms);
        $table->set('params', $this->params->toString());
        $table->store();
        return $webforms;
    }
    private function vine_ma_get_authtoken($apikey) {
        if($apikey === '' || $apikey == null)
            return null;
        $url = VINEHOST.'/api/rest/2.0/login';
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Content-type: application/json',
            'x-api-key:' . $apikey
        ));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $html = curl_exec($ch);
        if(!curl_errno($ch) && $html != 'Unauthorized') {
            curl_close($ch);
            return $html;
        }
        else {
            curl_close($ch);
            return null;
        }
    }
}


